System.register("../../src/helpers", [], function() {
  "use strict";
  var __moduleName = "../../src/helpers";
  function distance(x1, y1, x2, y2) {
    return Math.sqrt(((x1 - x2) * (x1 - x2)) + ((y1 - y2) * (y1 - y2)));
  }
  return {get distance() {
      return distance;
    }};
});
System.register("../../src/nutrients", [], function() {
  "use strict";
  var __moduleName = "../../src/nutrients";
  var distance = $traceurRuntime.assertObject(System.get("../../src/helpers")).distance;
  var Nutrient = function Nutrient(game, x, y) {
    $traceurRuntime.superCall(this, $Nutrient.prototype, "constructor", [game, x, y, 'nutrients']);
    this.scale.x = 0.75;
    this.scale.y = 0.75;
    this.anchor.x = 0.5;
    this.anchor.y = 0.5;
    this.rotation = Math.PI * Math.random() * 2;
    this.connected = false;
    this.inputEnabled = true;
    this.events.onInputDown.add(this.click, this);
    this.events.onInputOver.add(this.hover, this);
    this.events.onInputOut.add(this.out, this);
  };
  var $Nutrient = Nutrient;
  ($traceurRuntime.createClass)(Nutrient, {
    nearestRoot: function() {
      var $__0 = this;
      var nearest;
      var rootDist = 9999999;
      var walkRoots = (function(children) {
        for (var $__2 = children[Symbol.iterator](),
            $__3; !($__3 = $__2.next()).done; ) {
          var root = $__3.value;
          {
            if (root.children.length) {
              walkRoots(root.children);
            } else if (!root.target) {
              var dist = distance(root.startX, root.startY, $__0.x, $__0.y);
              if (dist < rootDist) {
                rootDist = dist;
                nearest = root;
              }
            }
          }
        }
      });
      walkRoots(this.game.plant.roots);
      return [nearest, rootDist];
    },
    click: function() {
      if (this.root)
        return;
      var nearest = this.nearestRoot();
      if (nearest[0]) {
        var localDist = nearest[1] / (this.game.width + this.game.height);
        var energyCost = parseInt(localDist * 200);
        var waterCost = parseInt(localDist * 100);
        if (this.game.stats.energy >= energyCost && this.game.stats.water >= waterCost) {
          nearest[0].setTarget(this);
          this.game.stats.energy -= energyCost;
          this.game.stats.water -= waterCost;
          this.game.add.audio('build').play();
        } else {
          this.game.add.audio('error').play();
        }
      }
    },
    hover: function() {
      var nearest = this.nearestRoot();
      if (nearest[0]) {
        var localDist = nearest[1] / (this.game.width + this.game.height);
        this.game.costText.setText(("Cost: \n" + parseInt(localDist * 200) + " energy\n" + parseInt(localDist * 100) + " water"));
      }
    },
    out: function() {
      this.game.costText.setText('');
    },
    updateDay: function() {
      if (!this.connected)
        return;
      this.scale.x *= 0.98;
      this.scale.y *= 0.98;
      if (this.scale.x < 0.05) {
        this.alive = false;
        this.game.stats[this.nutrientName + 'PerDay']--;
      }
    }
  }, {}, Phaser.Sprite);
  var Water = function Water(game, x, y) {
    $traceurRuntime.superCall(this, $Water.prototype, "constructor", [game, x, y]);
    this.tint = 0x4F94CF;
    this.nutrientName = 'water';
  };
  var $Water = Water;
  ($traceurRuntime.createClass)(Water, {}, {}, Nutrient);
  return {get Water() {
      return Water;
    }};
});
System.register("../../src/plant", [], function() {
  "use strict";
  var __moduleName = "../../src/plant";
  var distance = $traceurRuntime.assertObject(System.get("../../src/helpers")).distance;
  var Plant = function Plant(game) {
    $traceurRuntime.superCall(this, $Plant.prototype, "constructor", [game, undefined, 'plant', true]);
    this.stem = [new Stem(game)];
    this.stemBitmap = new Phaser.BitmapData(game, 'stem', game.width, game.height - game.height / 3);
    this.leafPositions = [];
    this.roots = [new Root(game), new Root(game), new Root(game)];
    this.rootBitmap = new Phaser.BitmapData(game, 'roots', game.width, game.height / 3);
    this.stemsGrowing = 1;
    this.rootsGrowing = 0;
    this.add(new Phaser.Sprite(game, 0, this.game.groundY, this.rootBitmap));
    this.add(new Phaser.Sprite(game, 0, 0, this.stemBitmap));
    this.drawRoots();
    this.drawStem();
  };
  var $Plant = Plant;
  ($traceurRuntime.createClass)(Plant, {
    drawRoots: function() {
      var ctx = this.rootBitmap.context;
      ctx.lineCap = 'round';
      ctx.lineJoin = 'round';
      ctx.clearRect(0, 0, this.game.width, this.game.height);
      ctx.moveTo(this.game.width / 2, 0);
      var totalRoots = 0;
      var usedRoots = 0;
      var drawChildren = (function(children) {
        for (var $__7 = children[Symbol.iterator](),
            $__8; !($__8 = $__7.next()).done; ) {
          var root = $__8.value;
          {
            ctx.strokeStyle = ("#" + (0xB78139 + parseInt(Math.random() * 100)).toString(16));
            ctx.lineWidth = 2 + 3 / root.depth;
            totalRoots++;
            if (!root.target)
              continue;
            else
              usedRoots++;
            ctx.beginPath();
            ctx.moveTo(root.startX, root.startY);
            var percent = root.progress / root.distance;
            if (percent < 1) {
              root.progress += 1;
              var angle = Math.atan2((root.endY - root.startY), (root.endX - root.startX));
              var toX = root.startX + Math.cos(angle) * root.distance * percent;
              var toY = root.startY + Math.sin(angle) * root.distance * percent;
              ctx.lineTo(toX, toY);
            } else {
              ctx.lineTo(root.endX, root.endY);
              root.split();
            }
            ctx.closePath();
            ctx.stroke();
            if (root.children.length)
              drawChildren(root.children);
          }
        }
      });
      drawChildren(this.roots);
      this.rootBitmap.dirty = true;
      this.game.textLines[1] = ("Roots used: " + usedRoots + "/" + totalRoots);
    },
    priceLeaf: function() {
      this.game.costText.setText(("Cost: \n" + 15 + " energy\n" + 10 + " water"));
    },
    growLeaf: function() {
      var energyCost = 15;
      var waterCost = 10;
      if (this.leafPositions.length === 0 || this.game.stats.energy < energyCost || this.game.stats.water < waterCost) {
        this.game.add.audio('error').play();
        return;
      }
      var leafPos = this.leafPositions.shift();
      var leaf = this.game.add.sprite(leafPos[0], leafPos[1], 'leaf');
      leaf.update = function() {
        if (this.baseScale < 0.4) {
          this.baseScale += 0.02;
        }
        this.scale.x = this.baseScale;
        this.scale.y = this.baseScale;
      };
      leaf.anchor.x = 1.0;
      leaf.anchor.y = 0.5;
      leaf.scale.y = 0.4;
      leaf.baseScale = 0;
      leaf.animSeed = Math.random();
      leaf.rotation = leafPos[2];
      leaf.bringToTop();
      this.game.add.audio('build').play();
      this.game.stats.leaves++;
      this.game.stats.waterPerDay--;
      this.game.stats.energyPerDay++;
      this.game.stats.energy -= energyCost;
      this.game.stats.water -= waterCost;
    },
    findStem: function(rootStem) {
      var depth = 5;
      var lowestStem;
      var findSplit = (function(stems) {
        for (var $__7 = stems[Symbol.iterator](),
            $__8; !($__8 = $__7.next()).done; ) {
          var stem = $__8.value;
          {
            if (stem.depth < depth && stem.children.length === 0) {
              depth = stem.depth;
              lowestStem = stem;
            }
            findSplit(stem.children);
          }
        }
      });
      findSplit(rootStem);
      return [lowestStem, depth];
    },
    priceStem: function() {
      var stem = this.findStem(this.stem);
      if (stem[0]) {
        this.game.costText.setText(("Cost: \n" + (5 + stem[1]) + " energy\n" + (5 + stem[1] * 2) + " water"));
      }
    },
    growStem: function() {
      console.log(this.stem);
      var stem = this.findStem(this.stem);
      var energyCost = 5 + stem[1];
      var waterCost = 5 + stem[1] * 2;
      this.stemsGrowing += 3;
      if (stem[0] && this.game.stats.energy >= energyCost && this.game.stats.water >= waterCost) {
        stem[0].split();
        this.game.stats.energy -= energyCost;
        this.game.stats.water -= waterCost;
        this.game.add.audio('build').play();
      } else {
        this.game.add.audio('error').play();
      }
    },
    drawStem: function() {
      var $__5 = this;
      var ctx = this.stemBitmap.context;
      ctx.lineCap = 'round';
      ctx.lineJoin = 'round';
      ctx.clearRect(0, 0, this.game.width, this.game.height);
      ctx.moveTo(this.game.width / 2, 0);
      var drawChildren = (function(children) {
        var numSplit = children.length;
        var i = 0;
        for (var $__7 = children[Symbol.iterator](),
            $__8; !($__8 = $__7.next()).done; ) {
          var stem = $__8.value;
          {
            ctx.strokeStyle = ("#08" + (160 - Math.random() * 20).toString(16).substr(0, 2) + "00");
            ctx.lineWidth = 2 + 4 / stem.depth;
            ctx.beginPath();
            ctx.moveTo(stem.startX, stem.startY);
            var finishGrowth = false;
            if (stem.progress < 1) {
              stem.progress += 0.05;
              if (stem.progress >= 1)
                finishGrowth = true;
            }
            var endX;
            var angle = Math.PI * (-numSplit / 2) * i;
            if (stem.depth === 1) {
              endX = stem.startX;
            } else {
              endX = stem.startX + Math.cos(angle) * stem.depth * 10 * stem.progress;
            }
            var yAdd = 35 * stem.progress;
            var endY = stem.startY - Math.sin(angle) * 20 * stem.progress;
            if (finishGrowth) {
              var leafX = stem.startX + Math.cos(angle) * stem.depth * 10 * 0.5;
              var leafY = stem.startY - Math.sin(angle) * 20 * 0.5 - 17.5;
              if (stem.depth == 1)
                leafX -= 4;
              $__5.leafPositions.push([leafX, leafY, Math.PI / 4 + (Math.random() - 0.5) * Math.PI / 2]);
              $__5.leafPositions.push([leafX, leafY, Math.PI - Math.PI / 4 + (Math.random() - 0.5) * Math.PI / 2]);
              $__5.stemsGrowing--;
            }
            ctx.lineTo(endX, endY - yAdd);
            if (stem.progress < 1) {
              stem.progress += 0.025;
            } else {
              stem.endX = endX;
              stem.endY = endY - yAdd;
            }
            ctx.closePath();
            ctx.stroke();
            if (stem.children.length)
              drawChildren(stem.children);
            i++;
          }
        }
      });
      drawChildren(this.stem);
      this.stemBitmap.dirty = true;
    }
  }, {}, Phaser.Group);
  var Stem = function Stem(game, parent) {
    this.game = game;
    this.parent = parent;
    this.children = [];
    if (parent) {
      this.startX = parent.endX;
      this.startY = parent.endY;
      this.depth = parent.depth + 1;
    } else {
      this.startX = game.width / 2;
      this.startY = game.height - game.height / 3;
      this.depth = 1;
    }
    this.progress = 0;
    this.done = false;
  };
  var $Stem = Stem;
  ($traceurRuntime.createClass)(Stem, {split: function() {
      console.log(this.children.length);
      if (this.progress >= 1 && this.children.length == 0) {
        this.children = [new $Stem(this.game, this), new $Stem(this.game, this), new $Stem(this.game, this)];
        this.done = true;
        return true;
      }
    }}, {});
  var Root = function Root(game, parent) {
    this.game = game;
    this.parent = parent;
    this.children = [];
    if (parent) {
      this.startX = parent.endX;
      this.startY = parent.endY;
      this.depth = parent.depth + 1;
    } else {
      this.startX = game.width / 2;
      this.startY = 0;
      this.depth = 1;
    }
    this.progress = 0;
    this.distance = 0;
  };
  var $Root = Root;
  ($traceurRuntime.createClass)(Root, {
    setTarget: function(nutrient) {
      if (this.target)
        return;
      this.target = nutrient;
      this.target.root = this;
      this.endX = nutrient.x;
      this.endY = nutrient.y;
      this.distance = distance(this.startX, this.startY, this.endX, this.endY);
      this.game.plant.rootsGrowing++;
    },
    split: function() {
      if (this.target.connected)
        return false;
      if (this.progress >= this.distance && this.children.length == 0) {
        this.target.connected = true;
        this.target.alive = true;
        this.children = [new $Root(this.game, this), new $Root(this.game, this)];
        this.game.plant.rootsGrowing--;
        this.game.stats.waterPerDay++;
      }
    }
  }, {});
  return {get Plant() {
      return Plant;
    }};
});
System.register("../../src/states/game", [], function() {
  "use strict";
  var __moduleName = "../../src/states/game";
  var Plant = $traceurRuntime.assertObject(System.get("../../src/plant")).Plant;
  var Water = $traceurRuntime.assertObject(System.get("../../src/nutrients")).Water;
  var GameState = function GameState() {
    $traceurRuntime.defaultSuperCall(this, $GameState.prototype, arguments);
  };
  var $GameState = GameState;
  ($traceurRuntime.createClass)(GameState, {
    create: function() {
      this.addText();
      this.game.stats = {
        day: 1,
        lastDay: new Date().getTime(),
        energy: 25,
        energyPerDay: 0,
        water: 25,
        waterPerDay: 0,
        leaves: 0,
        maxLeaves: 2
      };
      console.log('In game!');
      this.game.groundY = this.game.height - this.game.height / 3;
      this.game.add.tileSprite(0, this.game.groundY + 5, this.game.width, 300, 'dirt');
      this.game.add.tileSprite(0, this.game.groundY - 20, this.game.width, 64, 'grass');
      this.game.clouds = this.game.add.group(null, 'clouds', true);
      this.addClouds();
      this.game.plant = new Plant(this.game);
      this.game.nutrients = this.game.add.group(undefined, 'nutrients', true);
      this.game.nutrients.y = this.game.groundY;
      for (var i = 0; i < 50; i++) {
        this.addNutrient();
      }
      this.addButtons();
    },
    addText: function() {
      this.game.gameText = this.game.add.text(20, 20, '', {
        font: '18px Arial',
        fill: '#005713'
      });
      this.game.textLines = [];
      this.game.costText = this.game.add.text(300, 20, '', {
        font: '18px Arial',
        fill: '#005713'
      });
    },
    addClouds: function() {
      for (var i = 0; i < 15; i++) {
        var cl = this.game.add.sprite(Math.random() * 1280, Math.random() * 300 + 100, 'clouds');
        cl.speed = (Math.random() - 0.5) * 5;
        cl.anchor.x = 0.5;
        cl.anchor.y = 0.5;
        cl.scale.x = cl.speed / 3;
        cl.scale.y = cl.speed / 3;
        this.game.clouds.add(cl);
      }
    },
    addButtons: function() {
      var stemBtn = this.game.add.button(this.game.width - 276, 20, 'grow_stem', this.game.plant.growStem, this.game.plant, 1, 0, 2);
      stemBtn.onInputOver.add(this.game.plant.priceStem, this.game.plant);
      stemBtn.onInputOut.add(function() {
        this.game.costText.setText('');
      }, this);
      var leafBtn = this.game.add.button(this.game.width - 276, 104, 'grow_leaf', this.game.plant.growLeaf, this.game.plant, 1, 0, 2);
      leafBtn.onInputOver.add(this.game.plant.priceLeaf, this.game.plant);
      leafBtn.onInputOut.add(function() {
        this.game.costText.setText('');
      }, this);
      this.game.add.button();
    },
    update: function() {
      this.game.clouds.forEachExists(function(cloud) {
        cloud.x += cloud.speed;
        if (cloud.x < -200)
          cloud.x = this.game.width + 200;
        else if (cloud.x > this.game.width + 200)
          cloud.x = -200;
      }, this);
      if (this.game.stats.lastDay + 1000 < new Date().getTime()) {
        this.game.stats.day++;
        this.game.stats.lastDay = new Date().getTime();
        this.processDay();
        console.log('Day', this.game.day);
      }
      this.game.textLines[0] = ("Day: " + this.game.stats.day);
      if (this.game.plant.rootsGrowing > 0)
        this.game.plant.drawRoots();
      if (this.game.plant.stemsGrowing > 0)
        this.game.plant.drawStem();
      this.game.stats.leavesLeft = this.game.plant.leafPositions.length;
      this.game.textLines[2] = ("Leaves: " + this.game.stats.leaves + "/" + (this.game.stats.leaves + this.game.stats.leavesLeft));
      this.game.textLines[4] = ("Energy: " + this.game.stats.energy + " (" + this.game.stats.energyPerDay + " per day)");
      this.game.textLines[5] = ("Water: " + this.game.stats.water + " (" + this.game.stats.waterPerDay + " per day)");
      this.game.gameText.setText(this.game.textLines.join('\n'));
      if (this.game.stats.water < 0) {
        this.game.state.start('gameover');
      }
    },
    processDay: function() {
      this.game.stats.energy += this.game.stats.energyPerDay;
      this.game.nutrients.forEachAlive(function(nutrient) {
        nutrient.updateDay();
      });
      this.game.stats.water += this.game.stats.waterPerDay;
      if (Math.random() > 0.8) {
        this.addNutrient();
      }
    },
    addNutrient: function() {
      if (this.game.nutrients.length >= 250)
        return;
      this.game.nutrients.add(new Water(this.game, Math.random() * this.game.width, Math.random() * 170 + 20));
    }
  }, {}, Phaser.State);
  return {get GameState() {
      return GameState;
    }};
});
System.register("../../src/states/gameover", [], function() {
  "use strict";
  var __moduleName = "../../src/states/gameover";
  var GameOverState = function GameOverState() {
    $traceurRuntime.defaultSuperCall(this, $GameOverState.prototype, arguments);
  };
  var $GameOverState = GameOverState;
  ($traceurRuntime.createClass)(GameOverState, {
    preload: function() {
      console.log('Game over...', this.game);
      this.game.plant.destroy();
      this.game.clouds.destroy();
      this.game.nutrients.destroy();
      var rect = new Phaser.Rectangle(0, 0, this.game.width, this.game.height);
      var gameOverText = ("Game over!\n\nYour plant lived for " + this.game.stats.day + " days\n\nEnergy score: " + this.game.stats.energy);
      this.game.add.text(this.game.width / 2 - 200, 50, gameOverText, {
        font: '32px Arial',
        fill: '#005713',
        align: 'center'
      });
      this.game.add.button(this.game.width / 2 - 128, 300, 'tweet_score', this.tweet, this, 1, 0, 2);
      this.game.add.button(this.game.width / 2 - 128, 384, 'start_game', this.restart, this, 1, 0, 2);
      ga('send', 'event', 'LD29', ("Energy: " + this.game.stats.energy + ". Days: " + this.game.stats.day));
    },
    tweet: function() {
      var text = encodeURIComponent(("My plant lived for " + this.game.stats.day + " days and got an energy score of " + this.game.stats.energy + ". Can you beat me? #LD48"));
      window.open(("https://twitter.com/share?text=" + text + "&via=lindekleiv"));
      ga('send', 'event', 'LD29', 'Tweet');
    },
    restart: function() {
      window.location = '?autostart=true';
    }
  }, {}, Phaser.State);
  return {get GameOverState() {
      return GameOverState;
    }};
});
System.register("../../src/states/preload", [], function() {
  "use strict";
  var __moduleName = "../../src/states/preload";
  var PreloadState = function PreloadState() {
    $traceurRuntime.defaultSuperCall(this, $PreloadState.prototype, arguments);
  };
  var $PreloadState = PreloadState;
  ($traceurRuntime.createClass)(PreloadState, {
    preload: function() {
      var loadingText = this.game.add.text(this.game.width / 2 - 100, 65, 'Loading...', {
        font: '32px Arial',
        fill: '#005713',
        align: 'center'
      });
      console.log('Loading...', this.game);
      this.game.stage.backgroundColor = 0x8ADDFF;
      this.game.load.image('title_screen', 'img/title_screen.png');
      this.game.load.image('dirt', 'img/dirt.png');
      this.game.load.image('grass', 'img/grass.png');
      this.game.load.image('clouds', 'img/clouds.png');
      this.game.load.image('nutrients', 'img/nutrients.png');
      this.game.load.image('leaf', 'img/leaf.png');
      this.game.load.spritesheet('grow_stem', 'img/buttons/grow_stem.png', 256, 64);
      this.game.load.spritesheet('grow_leaf', 'img/buttons/grow_leaf.png', 256, 64);
      this.game.load.spritesheet('start_game', 'img/buttons/start_game.png', 256, 64);
      this.game.load.spritesheet('tweet_score', 'img/buttons/tweet_score.png', 256, 64);
      this.game.load.audio('error', 'sound/error.ogg', true);
      this.game.load.audio('build', 'sound/build.ogg', true);
      this.game.load.audio('music', 'sound/music.ogg', true);
      var finished = 0;
      this.load.onFileComplete.add(function() {
        finished++;
        loadingText.setText(("Loading " + finished + "/13"));
      }, this);
    },
    create: function() {
      this.game.state.start('splash');
    }
  }, {}, Phaser.State);
  return {get PreloadState() {
      return PreloadState;
    }};
});
System.register("../../src/states/splash", [], function() {
  "use strict";
  var __moduleName = "../../src/states/splash";
  var SplashState = function SplashState() {
    $traceurRuntime.defaultSuperCall(this, $SplashState.prototype, arguments);
  };
  var $SplashState = SplashState;
  ($traceurRuntime.createClass)(SplashState, {
    preload: function() {
      if (window.location.href.indexOf('autostart') != -1) {
        var music = this.game.add.audio('music');
        music.loop = true;
        music.play();
        this.game.state.start('game');
      }
      console.log('Splash...', this.game);
    },
    create: function() {
      var music = this.game.add.audio('music');
      music.loop = true;
      music.play();
      this.game.add.sprite(this.game.width / 2 - 310, 0, 'title_screen');
      this.game.add.button(this.game.width / 2 - 128, 190, 'start_game', this.startGame, this, 1, 0, 2);
      this.addInstructions();
    },
    startGame: function() {
      this.game.state.start('game');
    },
    addInstructions: function() {
      var instructions = "How to play:\n        - Grow your plant and harvest energy from the sun.\n        - Extend your roots to get more water.\n        - Leaves are expensive, but more leaves will give you more energy.\n        - Try to survive for as long as possible.\n\nAbout the game:\n        \"Root cause\" was made by Olav Lindekleiv (@lindekleiv on Twitter)\n        in 48 hours for Ludum Dare 29. Theme: Beneath the surface\n\nSoftware/libraries used:\n        Phaser.js, Traceur.js, WebStorm, GIMP, LMMS, Bfxr.net\n        ";
      this.game.add.text(this.game.width / 2 - 275, 475, instructions, {
        font: '16px Arial',
        fill: '#005713'
      });
    }
  }, {}, Phaser.State);
  return {get SplashState() {
      return SplashState;
    }};
});
System.register("../../src/main", [], function() {
  "use strict";
  var __moduleName = "../../src/main";
  var PreloadState = $traceurRuntime.assertObject(System.get("../../src/states/preload")).PreloadState;
  var SplashState = $traceurRuntime.assertObject(System.get("../../src/states/splash")).SplashState;
  var GameState = $traceurRuntime.assertObject(System.get("../../src/states/game")).GameState;
  var GameOverState = $traceurRuntime.assertObject(System.get("../../src/states/gameover")).GameOverState;
  var Game = function Game() {
    $traceurRuntime.superCall(this, $Game.prototype, "constructor", [document.body.clientWidth, document.body.clientHeight, Phaser.AUTO]);
    this.state.add('preload', PreloadState);
    this.state.add('splash', SplashState);
    this.state.add('game', GameState);
    this.state.add('gameover', GameOverState);
    this.state.start('preload');
  };
  var $Game = Game;
  ($traceurRuntime.createClass)(Game, {}, {}, Phaser.Game);
  window.onload = function() {
    var game = new Game();
  };
  return {};
});
System.get("../../src/main" + '');

//# sourceMappingURL=app.map
