import {distance} from './helpers';

class Nutrient extends Phaser.Sprite {
    constructor(game, x, y) {
        super(game, x, y, 'nutrients');
        this.scale.x = 0.75;
        this.scale.y = 0.75;
        this.anchor.x = 0.5;
        this.anchor.y = 0.5;
        this.rotation = Math.PI*Math.random()*2;
        this.connected = false;
        this.inputEnabled = true;
        this.events.onInputDown.add(this.click, this);
        this.events.onInputOver.add(this.hover, this);
        this.events.onInputOut.add(this.out, this);
    }

    nearestRoot() {
        var nearest;
        var rootDist = 9999999;
        var walkRoots = (children) => {
            for (var root of children) {
                if (root.children.length) {
                    walkRoots(root.children);

                } else if (!root.target) {
                    var dist = distance(root.startX, root.startY, this.x, this.y);
                    if (dist < rootDist) {
                        rootDist = dist;
                        nearest = root;
                    }
                }
            }
        };
        walkRoots(this.game.plant.roots);

        return [nearest, rootDist];
    }

    click() {
        if (this.root) return;

        var nearest = this.nearestRoot();
        if (nearest[0]) {
            var localDist = nearest[1] / (this.game.width + this.game.height);
            var energyCost = parseInt(localDist * 200);
            var waterCost = parseInt(localDist * 100);

            if (this.game.stats.energy >= energyCost && this.game.stats.water >= waterCost) {
                nearest[0].setTarget(this);
                this.game.stats.energy -= energyCost;
                this.game.stats.water -= waterCost;
                this.game.add.audio('build').play();
            } else {
                this.game.add.audio('error').play();
            }
        }
    }

    hover() {
        var nearest = this.nearestRoot();
        if (nearest[0]) {
            var localDist = nearest[1] / (this.game.width + this.game.height);
            this.game.costText.setText(`Cost: \n${parseInt(localDist * 200)} energy\n${parseInt(localDist * 100)} water`);
        }
    }

    out() {
        this.game.costText.setText('');
    }

    updateDay() {
        if (!this.connected) return;
        this.scale.x *= 0.98;
        this.scale.y *= 0.98;

        if (this.scale.x < 0.05) {
            this.alive = false;
            this.game.stats[this.nutrientName+'PerDay']--;
        }
    }
}

export class Water extends Nutrient {
    constructor(game, x, y) {
        super(game, x, y);
        this.tint = 0x4F94CF;
        this.nutrientName = 'water'
    }
}