import {Plant} from '../plant';
import {Water} from '../nutrients';

export class GameState extends Phaser.State {
    create() {
        this.addText();

        this.game.stats = {
            day: 1,
            lastDay: new Date().getTime(),
            energy: 25,
            energyPerDay:0,
            water: 25,
            waterPerDay:0,
            leaves: 0,
            maxLeaves: 2
        };

        console.log('In game!');
        this.game.groundY = this.game.height - this.game.height / 3;
        this.game.add.tileSprite(0, this.game.groundY + 5, this.game.width, 300, 'dirt');
        this.game.add.tileSprite(0, this.game.groundY - 20, this.game.width, 64, 'grass');

        this.game.clouds = this.game.add.group(null, 'clouds', true);
        this.addClouds();

        this.game.plant = new Plant(this.game);

        this.game.nutrients = this.game.add.group(undefined, 'nutrients', true);
        this.game.nutrients.y = this.game.groundY;
        for (var i = 0; i < 50; i++) {
            this.addNutrient();
        }

        this.addButtons();
    }

    addText() {
        this.game.gameText = this.game.add.text(20, 20, '', {
            font: '18px Arial',
            fill: '#005713'
        });
        this.game.textLines = [];

        this.game.costText = this.game.add.text(300, 20, '', {
            font: '18px Arial',
            fill: '#005713'
        });
    }

    addClouds() {
        for (var i = 0; i < 15; i++) {
            var cl = this.game.add.sprite(Math.random() * 1280, Math.random() * 300 + 100, 'clouds');
            cl.speed = (Math.random() - 0.5) * 5;
            cl.anchor.x = 0.5;
            cl.anchor.y = 0.5;
            cl.scale.x = cl.speed / 3;
            cl.scale.y = cl.speed / 3;
            this.game.clouds.add(cl);
        }
    }

    addButtons() {
        var stemBtn = this.game.add.button(this.game.width - 276, 20, 'grow_stem', this.game.plant.growStem, this.game.plant, 1, 0, 2);
        stemBtn.onInputOver.add(this.game.plant.priceStem, this.game.plant);
        stemBtn.onInputOut.add(function () {
            this.game.costText.setText('');
        }, this);

        var leafBtn = this.game.add.button(this.game.width - 276, 104, 'grow_leaf', this.game.plant.growLeaf, this.game.plant, 1, 0, 2);
        leafBtn.onInputOver.add(this.game.plant.priceLeaf, this.game.plant);
        leafBtn.onInputOut.add(function () {
            this.game.costText.setText('');
        }, this);
        this.game.add.button()
    }

    update() {
        //console.log(1)
        this.game.clouds.forEachExists(function (cloud) {
            cloud.x += cloud.speed;
            if (cloud.x < -200) cloud.x = this.game.width + 200;
            else if (cloud.x > this.game.width + 200) cloud.x = -200;
        }, this);

        if (this.game.stats.lastDay + 1000 < new Date().getTime()) {
            this.game.stats.day++;
            this.game.stats.lastDay = new Date().getTime();
            this.processDay();
            console.log('Day', this.game.day)
        }

        this.game.textLines[0] = `Day: ${this.game.stats.day}`;

        if(this.game.plant.rootsGrowing > 0) this.game.plant.drawRoots();
        if(this.game.plant.stemsGrowing > 0) this.game.plant.drawStem();

        this.game.stats.leavesLeft = this.game.plant.leafPositions.length;
        this.game.textLines[2] = `Leaves: ${this.game.stats.leaves}/${this.game.stats.leaves+this.game.stats.leavesLeft}`;

        this.game.textLines[4] = `Energy: ${this.game.stats.energy} (${this.game.stats.energyPerDay} per day)`;
        this.game.textLines[5] = `Water: ${this.game.stats.water} (${this.game.stats.waterPerDay} per day)`;

        this.game.gameText.setText(this.game.textLines.join('\n'));

        if(this.game.stats.water < 0) {
            this.game.state.start('gameover');
        }
    }

    processDay() {
        this.game.stats.energy += this.game.stats.energyPerDay;
        this.game.nutrients.forEachAlive(function (nutrient) {
            nutrient.updateDay();
        });

        this.game.stats.water += this.game.stats.waterPerDay;

        if (Math.random() > 0.8) {
            this.addNutrient();
        }
    }

    addNutrient() {
        if (this.game.nutrients.length >= 250) return;
        this.game.nutrients.add(new Water(this.game, Math.random() * this.game.width, Math.random() * 170 + 20));
    }
}

